/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* SPDX-License-Identifier: LGPL-2.1-or-later */
/*
 * libqrtr-glib -- GLib/GIO based library to control QRTR devices
 *
 * Copyright (C) 2020 Eric Caruso <ejcaruso@chromium.org>
 * Copyright (C) 2020 Aleksander Morgado <aleksander@aleksander.es>
 */

#ifndef _LIBQRTR_GLIB_H_
#define _LIBQRTR_GLIB_H_

#define __LIBQRTR_GLIB_H_INSIDE__

#include "qrtr-bus.h"
#include "qrtr-node.h"
#include "qrtr-client.h"
#include "qrtr-utils.h"

#endif /* _LIBQRTR_GLIB_H_ */
